@@ -1,6 +1,7 @@
 import os
 import argparse
 import platform
+import zipfile
 
 import reCBZ
 from reCBZ import wrappers, util
@@ -75,25 +76,30 @@ def main():
         action="store_const",
         help="disable all progress messages")
     # mode_group.add_argument( "-u", "--unpack",
-    #     const=0,
+    #     const='unpack',
     #     dest="mode",
     #     action="store_const",
     #     help="unpack the archive to the currect directory (safe)")
     mode_group.add_argument( "-c", "--compare",
-        const=1,
+        const='compare',
         dest="mode",
         action="store_const",
-        help="test a small sample with all formats and print the results (safe)")
+        help="calculate filesize when converting to each image format (safe)")
     mode_group.add_argument( "-a", "--assist",
-        const=2,
+        const='assist',
         dest="mode",
         action="store_const",
-        help="compare, then ask which format to use for a real run")
+        help="compare, then ask which format to repack with")
     mode_group.add_argument( "-A" ,"--auto",
-        const=3,
+        const='auto',
         dest="mode",
         action="store_const",
-        help="compare, then automatically picks the best format for a real run")
+        help="compare, then automatically pick the best format to repack with")
+    mode_group.add_argument( "-J" ,"--join",
+        const='join',
+        dest="mode",
+        action="store_const",
+        help="append the contents of each file to the first/leftmost file")
     fmt_group.add_argument( "--nowebp",
         default=Config.blacklistedfmts,
         const=f'{Config.blacklistedfmts} webp webpll',
@@ -118,6 +124,11 @@ def main():
         dest="outformat",
         action="store_const",
         help="save archive as cbz")
+    parser.add_argument( "--noprev",
+        default=False,
+        dest="noprev",
+        action="store_true",
+        help="ignore previously repacked files")
     parser.add_argument( "--compress",
         default=Config.compresszip,
         dest="compresszip",
@@ -126,7 +137,7 @@ def main():
     process_group.add_argument("--process",
         default=Config.processes,
         choices=(range(1,33)),
-        metavar="[1-32]",
+        metavar="1-32",
         dest="processes",
         type=int,
         help="maximum number of processes to spawn")
@@ -138,14 +149,14 @@ def main():
     fmt_group.add_argument( "--fmt",
         default=Config.imageformat,
         choices=('jpeg', 'png', 'webp', 'webpll'),
-        metavar="fmt",
+        metavar="format",
         dest="imageformat",
         type=str,
         help="format to convert pages to: jpeg, webp, webpll, or png")
     parser.add_argument( "--quality",
         default=Config.quality,
         choices=(range(1,101)),
-        metavar="[0-95]",
+        metavar="0-95",
         dest="quality",
         type=int,
         help="save quality for lossy formats. >90 not recommended")
@@ -155,30 +166,31 @@ def main():
         dest="resolution",
         type=str,
         help="rescale images to the specified resolution")
-    parser.add_argument( "-noup", "--noupscale",
+    parser.add_argument( "--noup",
         default=Config.noupscale,
         dest="noupscale",
         action="store_true",
         help="disable upscaling with --size")
-    parser.add_argument( "-nodw", "--nodownscale",
+    parser.add_argument( "--nodown",
         default=Config.nodownscale,
         dest="nodownscale",
         action="store_true",
         help="disable downscaling with --size")
-    parser.add_argument( "-bw", "--grayscale",
+    parser.add_argument( "--bw",
         default=Config.grayscale,
         dest="grayscale",
         action="store_true",
         help="convert images to grayscale")
-    parser.add_argument( "--version",
-        dest="show_version",
-        action="store_true",
-        help="show version and exit")
     parser.add_argument( "--config",
         dest="show_config",
         action="store_true",
         help="show current settings")
+    parser.add_argument( "--version",
+        dest="show_version",
+        action="store_true",
+        help="show version and exit")
     args, unknown_args = parser.parse_known_args()
+
     # this is probably not the most pythonic way to do this
     # I'm sorry guido-san...
     for key, val in args.__dict__.items():
@@ -206,30 +218,50 @@ def main():
             parser.print_help()
             print(f'\nunknown file or option: {arg}')
             exit(1)
+
     if len(paths) <= 0:
         print(f'{reCBZ.CMDNAME}: missing input file (see --help)')
         parser.print_usage()
         exit(1)
+
+    if args.mode == 'join':
+        if not len(paths) >= 2:
+            print(f'{reCBZ.CMDNAME}: join: at least two files are needed')
+            exit(1)
+
+    if args.noprev:
+        new = []
+        for filename in paths:
+            comment = zipfile.ZipFile(filename).comment
+            if not comment == str.encode(Config.ZIPCOMMENT):
+                new.append(filename)
+        diff = len(paths) - len(new)
+        if diff > 0:
+            print(f'{reCBZ.CMDNAME}: noprev: ignoring {diff} files')
+            if len(new) == 0:
+                exit(1)
+            else:
+                paths = new
+
     # everything passed
     if reCBZ.SHOWTITLE: print_title()
-    mode = args.mode
-    for filename in paths:
-        try:
-            if mode is None:
-                wrappers.repack_fp(filename)
-            elif mode == 0:
-                wrappers.unpack_fp(filename)
-            elif mode == 1:
-                wrappers.compare_fmts_fp(filename)
-            elif mode == 2:
-                wrappers.assist_repack_fp(filename)
-            elif mode == 3:
-                wrappers.auto_repack_fp(filename)
-        except InterruptedError:
-            continue
-        except (KeyboardInterrupt, util.MPrunnerInterrupt):
-            print('\nGoooooooooodbye')
-            exit(1)
+    try:
+        if args.mode == 'join':
+            wrappers.join_fps(paths[0], paths[1:])
+        for filename in paths:
+                if args.mode is None:
+                    wrappers.repack_fp(filename)
+                elif args.mode == 'unpack':
+                    wrappers.unpack_fp(filename)
+                elif args.mode == 'compare':
+                    wrappers.compare_fmts_fp(filename)
+                elif args.mode == 'assist':
+                    wrappers.assist_repack_fp(filename)
+                elif args.mode == 'auto':
+                    wrappers.auto_repack_fp(filename)
+    except (KeyboardInterrupt, util.MPrunnerInterrupt):
+        print('\nGoooooooooodbye')
+        exit(1)
 
 
 if __name__ == '__main__':
