@@ -12,7 +12,7 @@ from PIL import Image
 
 from reCBZ.formats import *
 from reCBZ.config import Config
-from reCBZ.util import mylog, MP_run_tasks, SIGNINT_ctrl_c
+from reCBZ.util import mylog, MP_run_tasks, SIGNINT_ctrl_c, human_sort
 
 # TODO:
 # include docstrings
@@ -86,10 +86,11 @@ class Page():
 
 
 class Archive():
-    source_id:str = 'Source'
-    new_id:str = ' [reCBZ]'
-    temp_prefix:str = f'reCBZCACHE_'
+    _SOURCE_NAME:str = 'Source'
+    _CACHE_PREFIX:str = 'reCBZCACHE_'
+    _global_cache_id:str = f'{_CACHE_PREFIX}{Config.tempuuid}_'
     validbookformats:tuple = ('cbz', 'zip', 'epub', 'mobi')
+    chapter_prefix:str = 'v' # :) :D C:
 
     def __init__(self, filename:str):
         mylog('Archive: __init__')
@@ -111,22 +112,37 @@ class Archive():
         self._pages_nodown = Config.nodownscale
         self._pages_filter = Config.resamplemethod
         self._index:list = []
-        self.tempdir:Path = Path('.')
+        self._chapter_lengths = []
+        self._chapters = []
+        self._cachedir:Path = Path('.')
 
     def fetch_pages(self):
         if len(self._index) == 0:
             self._index = list(self.extract())
         return self._index
 
-    def extract(self, count:int=0, raw:bool=False) -> tuple:
-        # check and clean previous tempdirs
-        prev_dirs = Path(tempfile.gettempdir()).glob(f'{self.temp_prefix}*')
-        for path in prev_dirs:
-            assert path != tempfile.gettempdir() # for the love of god
-            mylog(f'{path} exists, cleaning up')
-            shutil.rmtree(path)
+    def fetch_chapters(self):
+        # ensure it's a copy, so we can't delete the actual objects
+        index_copy = [page for page in self.fetch_pages()]
+        if len(self._chapter_lengths) == 0:
+            self._chapter_lengths = [len(index_copy)]
+        chapters = []
+        for length in self._chapter_lengths:
+            chapters.append(index_copy[:length])
+            del index_copy[:length]
+        return chapters
 
-        self.tempdir = Path(tempfile.mkdtemp(prefix=f'{self.temp_prefix}'))
+    def extract(self, count:int=0, raw:bool=False) -> tuple:
+        # check and clean previous cache
+        tempdir = Path(tempfile.gettempdir())
+        prev_dirs = tempdir.glob(f'{Archive._CACHE_PREFIX}*')
+        for path in prev_dirs:
+            assert path != tempdir # for the love of god
+            if not str(Archive._global_cache_id) in str(path):
+                mylog(f'hex {Archive._global_cache_id} not in {path}, cleaning up')
+                shutil.rmtree(path)
+
+        self._cachedir = Path(tempfile.mkdtemp(prefix=Archive._global_cache_id))
         try:
             source_zip = ZipFile(self.source_path)
         except BadZipFile as err:
@@ -143,16 +159,98 @@ class Archive():
 
         mylog(f'Extracting: {self.source_path}', progress=True)
         for file in compressed_files:
-            source_zip.extract(file, self.tempdir)
+            source_zip.extract(file, self._cachedir)
 
         # god bless you Georgy https://stackoverflow.com/a/50927977/
-        raw_files = tuple(filter(Path.is_file, Path(self.tempdir).rglob('*')))
-        pages = tuple(Page(path) for path in raw_files)
+        raw_paths = tuple(filter(Path.is_file, Path(self._cachedir).rglob('*')))
+        pages = tuple(Page(path) for path in raw_paths)
 
         mylog('', progress=True)
-        if raw: return raw_files
+        if raw: return raw_paths
         else: return pages
 
+    def add_chapter(self, second_archive, start=None, end=None) -> tuple:
+        try:
+            assert isinstance(second_archive, Archive)
+        except AssertionError:
+            raise ValueError('second_archive is not an instance of Archive')
+
+        new_chapter = second_archive.fetch_pages()
+        if start:
+            try:
+                assert type(start) is int
+            except AssertionError:
+                raise ValueError('start must be an integer')
+            # can raise IndexError but we want it to do so
+            new_chapter = new_chapter[start:]
+        if end:
+            try:
+                assert type(start) is int
+            except AssertionError:
+                raise ValueError('end must be an integer')
+            new_chapter = new_chapter[:end]
+
+        # ensure chapter1 is populated
+        self.fetch_chapters()
+        self._chapter_lengths.append(len(new_chapter))
+        self._index.extend(new_chapter)
+        return tuple(self.fetch_pages())
+
+    def convert_pages(self, fmt=None, quality=None, grayscale=None, size=None) -> tuple:
+        if fmt is not None: self._pages_format = fmt
+        if quality is not None: self._pages_quality = int(quality)
+        if grayscale is not None: self._pages_bw = bool(grayscale)
+        if size is not None: self._pages_size = size
+
+        source_pages = self.fetch_pages()
+        if self.opt_parallel:
+            results = MP_run_tasks(self._convert_page, source_pages)
+        else:
+            results = map(self._convert_page, source_pages)
+        self._index = [page for page in results if page]
+        return tuple(self._index)
+
+    def compute_fmt_sizes(self) -> tuple:
+        def compute_single_fmt(sample_pages, cachedir, fmt) -> tuple:
+            fmtdir = Path.joinpath(cachedir, fmt.name)
+            Path.mkdir(fmtdir)
+
+            pfunc = partial(self._convert_page, savedir=fmtdir, format=fmt)
+            if self.opt_parallel:
+                results = MP_run_tasks(pfunc, sample_pages)
+            else:
+                results = map(pfunc, sample_pages)
+
+            converted_pages = [page for page in results if page]
+            nbytes = sum(page.fp.stat().st_size for page in converted_pages)
+            return nbytes, fmt.desc, fmt.name
+
+        # extract images and compute their original size
+        # manually call extract so we don't overwrite _pages cache
+        source_pages = self.extract(count=self._fmt_samples)
+        nbytes = sum(page.fp.stat().st_size for page in source_pages)
+        source_fmt = source_pages[0].fmt
+        source_fsize = [nbytes, f'{Archive._SOURCE_NAME} ({source_fmt.desc})',
+                        source_fmt.name]
+
+        # compute the size of each format after converting.
+        # one thread per individual format. n processes per thread
+        fmt_fsizes = []
+        pfunc = partial(compute_single_fmt, source_pages, self._cachedir)
+        if self.opt_parallel:
+            with ThreadPool(processes=len(self._valid_page_formats)) as Tpool:
+                fmt_fsizes.extend(Tpool.map(pfunc, self._valid_page_formats))
+        else:
+            fmt_fsizes.extend(map(pfunc, self._valid_page_formats))
+
+        # finally, compare
+        # in multidepth lists, sorted compares the first element by default :)
+        sorted_fmts = list(sorted(fmt_fsizes))
+        sorted_fmts.insert(0, source_fsize)
+        mylog(str(sorted_fmts))
+        mylog('', progress=True)
+        return tuple(sorted_fmts)
+
     def write_archive(self, book_format='cbz', file_name:str='') -> str:
         if book_format not in Archive.validbookformats:
             raise ValueError(f"Invalid format '{book_format}'")
@@ -181,83 +279,52 @@ class Archive():
         else:
             raise ValueError
 
-    def convert_pages(self, fmt=None, quality=None, grayscale=None, size=None) -> tuple:
-        if fmt is not None: self._pages_format = fmt
-        if quality is not None: self._pages_quality = int(quality)
-        if grayscale is not None: self._pages_bw = bool(grayscale)
-        if size is not None: self._pages_size = size
-
-        source_pages = self.fetch_pages()
-        if self.opt_parallel:
-            results = MP_run_tasks(self._convert_page, source_pages)
-        else:
-            results = map(self._convert_page, source_pages)
-        self._index = [page for page in results if page]
-        return tuple(self._index)
-
-    def compute_fmt_sizes(self) -> tuple:
-        def compute_single_fmt(sample_pages, tempdir, fmt) -> tuple:
-            fmtdir = Path.joinpath(tempdir, fmt.name)
-            Path.mkdir(fmtdir)
-
-            pfunc = partial(self._convert_page, savedir=fmtdir, format=fmt)
-            if self.opt_parallel:
-                results = MP_run_tasks(pfunc, sample_pages)
-            else:
-                results = map(pfunc, sample_pages)
-
-            converted_pages = [page for page in results if page]
-            nbytes = sum(page.fp.stat().st_size for page in converted_pages)
-            return nbytes, fmt.desc, fmt.name
-
-        # extract images and compute their original size
-        # manually call extract so we don't overwrite _pages cache
-        source_pages = self.extract(count=self._fmt_samples)
-        nbytes = sum(page.fp.stat().st_size for page in source_pages)
-        source_fmt = source_pages[0].fmt
-        source_fsize = [nbytes, f'{Archive.source_id} ({source_fmt.desc})',
-                        source_fmt.name]
-
-        # compute the size of each format after converting.
-        # one thread per individual format. n processes per thread
-        fmt_fsizes = []
-        pfunc = partial(compute_single_fmt, source_pages, self.tempdir)
-        if self.opt_parallel:
-            with ThreadPool(processes=len(self._valid_page_formats)) as Tpool:
-                fmt_fsizes.extend(Tpool.map(pfunc, self._valid_page_formats))
-        else:
-            fmt_fsizes.extend(map(pfunc, self._valid_page_formats))
-
-        # finally, compare
-        # in multidepth lists, sorted compares the first element by default :)
-        sorted_fmts = list(sorted(fmt_fsizes))
-        sorted_fmts.insert(0, source_fsize)
-        mylog(str(sorted_fmts))
-        mylog('', progress=True)
-        return tuple(sorted_fmts)
-
     def _write_zip(self, savepath):
         new_zip = ZipFile(savepath,'w')
-        for page in self.fetch_pages():
-            try:
-                dest = page.fp.relative_to(self.tempdir)
+        chapters = self.fetch_chapters()
+        lead_zeroes = len(str(len(chapters)))
+        for i, chapter in enumerate(chapters):
+            for page in chapter:
+                # TODO this loop might be moved to a distinct function in the
+                # future, but essentially what we're trying to achieve is determine
+                # a page's path relative to its *local* cachedir, which varies by
+                # class instance (so they don't mix) but is constant to each process
+                temp = Path(tempfile.gettempdir())
+                proc_cache = temp.glob(f'{Archive._global_cache_id}*')
+                local_parent_dir = None
+                for path in proc_cache:
+                    if page.fp.is_relative_to(path):
+                        local_parent_dir = path
+                if local_parent_dir is None:
+                    raise OSError(f'{page.fp} not in any subpath of process cache')
+
+                dest = ''
+                if len(chapters) > 1: # no parent if there's only one chapter
+                    dest += f'{Archive.chapter_prefix}{i+1:0{lead_zeroes}d}/'
+                dest += f'{page.fp.relative_to(local_parent_dir)}'
+                dest = Path(dest)
                 if self._zip_compress:
                     new_zip.write(page.fp, dest, ZIP_DEFLATED, 9)
                 else:
                     new_zip.write(page.fp, dest, ZIP_STORED)
-            except ValueError:
-                msg = 'Path is being screwy. Does tempdir exist? '
-                msg += str(self.tempdir.exists())
-                msg += '\nwe might not have joined paths correctly in trans img'
-                raise ValueError(msg)
+        new_zip.comment = str.encode(Config.ZIPCOMMENT)
         new_zip.close()
         return savepath
 
     def _write_epub(self, savepath):
-        from reCBZ.epub import single_volume_epub
+        from reCBZ import epub
         title = self._source_stem
         mylog(f'Write .epub: {title}.epub', progress=True)
-        savepath = single_volume_epub(title, self.fetch_pages())
+        # solves the need to invert. critical for windows, because files
+        # aren't remotely ordered for whatever reason.
+        # TODO for the love of god find a more elegant solution
+        paths = human_sort([page.fp for page in self._index])
+        self._index = [Page(path) for path in paths]
+        chapters = self.fetch_chapters()
+        if len(chapters) > 1:
+            savepath = epub.multi_chapter_epub(title, chapters)
+        else:
+            savepath = epub.single_chapter_epub(title, self.fetch_pages())
         return savepath
 
     def _write_mobi(self, savepath):
@@ -375,3 +442,15 @@ class Archive():
             return newsize
         except (ValueError, AssertionError):
             return default_value
+
+    @classmethod
+    def cleanup(cls):
+        tempdir = Path(tempfile.gettempdir())
+        prev_dirs = tempdir.glob(f'{Archive._CACHE_PREFIX}*')
+        for path in prev_dirs:
+            assert path != tempdir # for the love of god
+            mylog(f'cleanup(): {path}]')
+            try:
+                shutil.rmtree(path)
+            except PermissionError:
+                mylog(f"PermissionError, couldn't clean {path}")
