@@ -6,6 +6,7 @@ import reCBZ
 
 
 class Config():
+    tempuuid:str = reCBZ.TEMPUUID
     overwrite:bool = reCBZ.OVERWRITE
     ignore:bool = reCBZ.IGNORE
     nowrite:bool = reCBZ.NOWRITE
@@ -24,6 +25,7 @@ class Config():
     grayscale:bool = reCBZ.GRAYSCALE
     # LANCZOS sacrifices performance for optimal upscale quality
     resamplemethod = Image.Resampling.LANCZOS
+    ZIPCOMMENT:str = 'repacked with reCBZ'
 
     @classmethod
     def pcount(cls) -> int:
@@ -44,13 +46,13 @@ class Config():
     @classmethod
     def term_width(cls) -> int:
         # limit output message width. ignored if verbose
-        TERM_COLUMNS, TERM_LINES = os.get_terminal_size()
         try:
+            TERM_COLUMNS, TERM_LINES = os.get_terminal_size()
             assert TERM_COLUMNS > 0 and TERM_LINES > 0
             if TERM_COLUMNS > 120: max_width = 120
             elif TERM_COLUMNS < 30: max_width = 30
             else: max_width = TERM_COLUMNS - 2
-        except AssertionError:
+        except (AssertionError, OSError):
             print("[!] Can't determine terminal size, defaulting to 78 cols")
             max_width = 78
         return max_width
