@@ -16,10 +16,11 @@ class Jpeg(LossyFmt):
     name:str = 'jpeg'
     ext:tuple = '.jpeg', '.jpg'
     desc:str = 'JPEG'
+    mime:str = 'image/jpeg'
 
     @classmethod
     def save(cls, img:Image.Image, dest):
-        img.save(dest, optimize=True, quality=cls.quality)
+        img.save(dest, format='JPEG', optimize=True, quality=cls.quality)
 
 
 class WebpLossy(LossyFmt):
@@ -28,47 +29,31 @@ class WebpLossy(LossyFmt):
     name:str = 'webp'
     ext:tuple = '.webp',
     desc:str = 'WebP'
+    mime:str = 'image/webp'
 
     @classmethod
     def save(cls, img:Image.Image, dest):
-        img.save(dest, lossless=cls.lossless, method=5, quality=cls.quality)
+        img.save(dest, fomrat='WEBP', lossless=cls.lossless, method=5, quality=cls.quality)
 
 
 class WebpLossless(LosslessFmt):
     name:str = 'webpll'
     ext:tuple = '.webp',
     desc:str = 'WebP Lossless'
+    mime:str = 'image/webp'
 
     @classmethod
     def save(cls, img:Image.Image, dest):
         # for some reason 'quality' is akin to Png compress_level when lossless
-        img.save(dest, lossless=cls.lossless, method=4, quality=100)
+        img.save(dest, format='WEBP', lossless=cls.lossless, method=4, quality=100)
 
 
 class Png(LosslessFmt):
     name:str = 'png'
     ext:tuple = '.png',
     desc:str = 'PNG'
+    mime:str = 'image/png'
 
     @classmethod
     def save(cls, img:Image.Image, dest):
-        img.save(dest, optimize=True, compress_level=9)
-
-
-def determine_format(img:Image.Image):
-    PIL_fmt = img.format
-    if PIL_fmt is None:
-        raise KeyError(f"Image.format returned None")
-    elif PIL_fmt == "PNG":
-        return Png
-    elif PIL_fmt == "JPEG":
-        return Jpeg
-    elif PIL_fmt == "WEBP":
-        # https://github.com/python-pillow/Pillow/discussions/6716
-        with open(img.filename, "rb") as fp:
-            if fp.read(16)[-1:] == b"L":
-                return WebpLossless
-            else:
-                return WebpLossy
-    else:
-        raise KeyError(f"'{PIL_fmt}': invalid format")
+        img.save(dest, format='PNG', optimize=True, compress_level=9)
